package net.pdynet.odbcproxyclient;

import java.io.IOException;
import java.net.URISyntaxException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.HttpClient;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.conn.ssl.X509HostnameVerifier;
import org.apache.http.impl.client.DecompressingHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.BasicClientConnectionManager;
import org.apache.http.params.CoreConnectionPNames;

public class HttpClientHelper {

	/**
	 * Vraci novou instanci {@link org.apache.http.client.HttpClient HTTP klienta},
	 * ktera neoveruje certifkacni autority pro HTTPS.
	 * Timeout pro spojeni a cekani na data nastavi na 30 sekund.
	 * 
	 * @return
	 * @throws URISyntaxException
	 * @throws KeyManagementException
	 * @throws UnrecoverableKeyException
	 * @throws NoSuchAlgorithmException
	 * @throws KeyStoreException
	 */
	public HttpClient getHttpClient()
			throws URISyntaxException, KeyManagementException, UnrecoverableKeyException,
			NoSuchAlgorithmException, KeyStoreException {
		//
		return getHttpClient("TLS");
	}
	
	
	/**
	 * Vraci novou instanci {@link org.apache.http.client.HttpClient HTTP klienta},
	 * ktera neoveruje certifkacni autority pro HTTPS.
	 * Timeout pro spojeni a cekani na data nastavi na 30 sekund.
	 * 
	 * @param protocol Protokol pro HTTPS. Napr. <tt>TLS</tt> nebo pro upresneni verze napr. <tt>TLSv1</tt>.
	 * @return
	 * @throws URISyntaxException
	 * @throws KeyManagementException
	 * @throws UnrecoverableKeyException
	 * @throws NoSuchAlgorithmException
	 * @throws KeyStoreException
	 */
	public HttpClient getHttpClient(String protocol)
			throws URISyntaxException, KeyManagementException, UnrecoverableKeyException,
			NoSuchAlgorithmException, KeyStoreException {
		
		// Inicializace HTTP klienta. Povoleni HTTPS i s neznamymi autoritami.
		/*
		TrustStrategy acceptingTrustStrategy = new TrustStrategy() {
			public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
				return true;
			}
		};
		*/
		
		if (StringUtils.isBlank(protocol))
			protocol = "TLS";
		
		SSLContext ctx = SSLContext.getInstance(protocol);
		
		X509TrustManager tm = new X509TrustManager() {
		    public void checkClientTrusted(X509Certificate[] xcs, String string) throws CertificateException {
		    }
		    public void checkServerTrusted(X509Certificate[] xcs, String string) throws CertificateException {
		    }
		    public X509Certificate[] getAcceptedIssuers() {
		        return null;
		    }
		};
		
        X509HostnameVerifier verifier = new X509HostnameVerifier() {
			public boolean verify(String arg0, SSLSession arg1) {
				return true;
			}
			public void verify(String host, SSLSocket ssl) throws IOException {
			}
			public void verify(String host, X509Certificate cert)
					throws SSLException {
			}
			public void verify(String host, String[] cns, String[] subjectAlts)
					throws SSLException {
			}
        };
		
		
		ctx.init(null, new TrustManager[]{tm}, null);
		
		/*
		SSLSocketFactory sf = new SSLSocketFactory(acceptingTrustStrategy,
				SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
		*/
		
		SSLSocketFactory sf = new SSLSocketFactory(ctx, verifier);
		
		SchemeRegistry registry = new SchemeRegistry();
		registry.register(new Scheme("https", 443, sf));
		registry.register(new Scheme("http", 80, PlainSocketFactory.getSocketFactory()));
		
		ClientConnectionManager ccm = new BasicClientConnectionManager(registry);
		
		// Timeout v ms
		int timeout = 30000;
		
		DefaultHttpClient dhc = new DefaultHttpClient(ccm);
		dhc.getParams().setParameter(CoreConnectionPNames.CONNECTION_TIMEOUT, timeout);
		dhc.getParams().setParameter(CoreConnectionPNames.SO_TIMEOUT, timeout);
		
		HttpClient httpClient = new DecompressingHttpClient(dhc);
		
		return httpClient;
	}
	
}
